@@ -8,12 +8,16 @@ use twilight_model::{
             InteractionChannel, InteractionMember,
         },
     },
+    channel::Attachment,
     guild::Role,
-    id::{ChannelId, GenericId, RoleId, UserId},
+    id::{
+        marker::{AttachmentMarker, ChannelMarker, GenericMarker, RoleMarker, UserMarker},
+        Id,
+    },
     user::User,
 };
 
-use crate::error::{ParseError, ParseOptionErrorType};
+use crate::error::{ParseError, ParseOptionError, ParseOptionErrorType};
 
 use super::internal::CommandOptionData;
 
@@ -25,9 +29,9 @@ use super::internal::CommandOptionData;
 ///
 /// ## Command models
 /// This trait can be implemented on structs representing a slash command
-/// model. All type fields must implement the [`CommandOption`] trait. See
-/// the [module documentation](crate::command) for a full list of supported
-/// types.
+/// model. All type fields must implement the [`CommandOption`] trait. A
+/// unit struct can be used if the command has no options. See the
+/// [module documentation](crate::command) for a full list of supported types.
 ///
 /// ```
 /// use twilight_interactions::command::{CommandModel, ResolvedUser};
@@ -127,6 +131,12 @@ pub trait CommandModel: Sized {
     fn from_interaction(data: CommandInputData) -> Result<Self, ParseError>;
 }
 
+impl CommandModel for Vec<CommandDataOption> {
+    fn from_interaction(data: CommandInputData) -> Result<Self, ParseError> {
+        Ok(data.options)
+    }
+}
+
 /// Parse command option into a concrete type.
 ///
 /// This trait is used by the implementation of [`CommandModel`] generated
@@ -142,6 +152,10 @@ pub trait CommandModel: Sized {
 /// the `value` attribute. In the example below, the inferred type would
 /// be `INTEGER`.
 ///
+/// A `value` method is also generated for each variant to obtain the
+/// value of the variant. This method is not described in the trait
+/// as it is only implemented for option choices.
+///
 /// ### Example
 /// ```
 /// use twilight_interactions::command::CommandOption;
@@ -155,6 +169,8 @@ pub trait CommandModel: Sized {
 ///     #[option(name = "Day", value = 86400)]
 ///     Day
 /// }
+///
+/// assert_eq!(TimeUnit::Minute.value(), 60);
 /// ```
 ///
 /// ### Macro attributes
@@ -187,6 +203,79 @@ pub struct CommandInputData<'a> {
 }
 
 impl<'a> CommandInputData<'a> {
+    /// Parse a field from the command data.
+    ///
+    /// This method can be used to manually parse a field from
+    /// raw data, for example with guild custom commands. The
+    /// method return [`None`] if the field is not present instead
+    /// of returning an error.
+    ///
+    /// ### Example
+    /// ```
+    /// use twilight_interactions::command::CommandInputData;
+    /// # use twilight_model::application::interaction::application_command::{CommandDataOption, CommandOptionValue};
+    /// #
+    /// # let options = vec![CommandDataOption { name: "message".into(), value: CommandOptionValue::String("Hello world".into()), focused: false }];
+    ///
+    /// // `options` is a Vec<CommandDataOption>
+    /// let data = CommandInputData { options, resolved: None };
+    /// let message = data.parse_field::<String>("message").unwrap();
+    ///
+    /// assert_eq!(message, Some("Hello world".to_string()));
+    /// ```
+    pub fn parse_field<T>(&self, name: &str) -> Result<Option<T>, ParseError>
+    where
+        T: CommandOption,
+    {
+        // Find command option value
+        let value = match self
+            .options
+            .iter()
+            .find(|option| option.name == name)
+            .map(|option| &option.value)
+        {
+            Some(value) => value.clone(),
+            None => return Ok(None),
+        };
+
+        // Parse command value
+        match CommandOption::from_option(
+            value,
+            CommandOptionData::default(),
+            self.resolved.as_deref(),
+        ) {
+            Ok(value) => Ok(Some(value)),
+            Err(kind) => Err(ParseError::Option(ParseOptionError {
+                field: name.to_string(),
+                kind,
+            })),
+        }
+    }
+
+    /// Get the name of the focused field.
+    ///
+    /// This method is useful when parsing commands with multiple
+    /// autocomplete fields.
+    ///
+    /// ### Example
+    /// ```
+    /// use twilight_interactions::command::CommandInputData;
+    /// # use twilight_model::application::interaction::application_command::{CommandDataOption, CommandOptionValue};
+    /// #
+    /// # let options = vec![CommandDataOption { name: "message".into(), value: CommandOptionValue::String("Hello world".into()), focused: true }];
+    ///
+    /// // `options` is a Vec<CommandDataOption>
+    /// let data = CommandInputData { options, resolved: None };
+    ///
+    /// assert_eq!(data.focused(), Some("message"));
+    /// ```
+    pub fn focused(&self) -> Option<&str> {
+        self.options
+            .iter()
+            .find(|option| option.focused)
+            .map(|option| &*option.name)
+    }
+
     /// Parse a subcommand [`CommandOptionValue`].
     ///
     /// This method signature is the same as the [`CommandOption`] trait,
@@ -234,10 +323,20 @@ macro_rules! lookup {
     ($resolved:ident.$cat:ident, $id:expr) => {
         $resolved
             .and_then(|resolved| resolved.$cat.get(&$id).cloned())
-            .ok_or_else(|| ParseOptionErrorType::LookupFailed($id.0))
+            .ok_or_else(|| ParseOptionErrorType::LookupFailed($id.get()))
     };
 }
 
+impl CommandOption for CommandOptionValue {
+    fn from_option(
+        value: CommandOptionValue,
+        _data: CommandOptionData,
+        _resolved: Option<&CommandInteractionDataResolved>,
+    ) -> Result<Self, ParseOptionErrorType> {
+        Ok(value)
+    }
+}
+
 impl CommandOption for String {
     fn from_option(
         value: CommandOptionValue,
@@ -328,7 +427,7 @@ impl CommandOption for bool {
     }
 }
 
-impl CommandOption for UserId {
+impl CommandOption for Id<UserMarker> {
     fn from_option(
         value: CommandOptionValue,
         _data: CommandOptionData,
@@ -341,7 +440,7 @@ impl CommandOption for UserId {
     }
 }
 
-impl CommandOption for ChannelId {
+impl CommandOption for Id<ChannelMarker> {
     fn from_option(
         value: CommandOptionValue,
         _data: CommandOptionData,
@@ -354,7 +453,7 @@ impl CommandOption for ChannelId {
     }
 }
 
-impl CommandOption for RoleId {
+impl CommandOption for Id<RoleMarker> {
     fn from_option(
         value: CommandOptionValue,
         _data: CommandOptionData,
@@ -367,7 +466,7 @@ impl CommandOption for RoleId {
     }
 }
 
-impl CommandOption for GenericId {
+impl CommandOption for Id<GenericMarker> {
     fn from_option(
         value: CommandOptionValue,
         _data: CommandOptionData,
@@ -380,13 +479,41 @@ impl CommandOption for GenericId {
     }
 }
 
+impl CommandOption for Id<AttachmentMarker> {
+    fn from_option(
+        value: CommandOptionValue,
+        _data: CommandOptionData,
+        _resolved: Option<&CommandInteractionDataResolved>,
+    ) -> Result<Self, ParseOptionErrorType> {
+        match value {
+            CommandOptionValue::Attachment(value) => Ok(value),
+            other => Err(ParseOptionErrorType::InvalidType(other.kind())),
+        }
+    }
+}
+
+impl CommandOption for Attachment {
+    fn from_option(
+        value: CommandOptionValue,
+        _data: CommandOptionData,
+        resolved: Option<&CommandInteractionDataResolved>,
+    ) -> Result<Self, ParseOptionErrorType> {
+        let attachment_id = match value {
+            CommandOptionValue::Attachment(value) => value,
+            other => return Err(ParseOptionErrorType::InvalidType(other.kind())),
+        };
+
+        lookup!(resolved.attachments, attachment_id)
+    }
+}
+
 impl CommandOption for User {
     fn from_option(
         value: CommandOptionValue,
         _data: CommandOptionData,
         resolved: Option<&CommandInteractionDataResolved>,
     ) -> Result<Self, ParseOptionErrorType> {
-        let user_id: UserId = match value {
+        let user_id = match value {
             CommandOptionValue::User(value) => value,
             other => return Err(ParseOptionErrorType::InvalidType(other.kind())),
         };
@@ -401,7 +528,7 @@ impl CommandOption for ResolvedUser {
         _data: CommandOptionData,
         resolved: Option<&CommandInteractionDataResolved>,
     ) -> Result<Self, ParseOptionErrorType> {
-        let user_id: UserId = match value {
+        let user_id = match value {
             CommandOptionValue::User(value) => value,
             other => return Err(ParseOptionErrorType::InvalidType(other.kind())),
         };
@@ -438,7 +565,7 @@ impl CommandOption for Role {
         _data: CommandOptionData,
         resolved: Option<&CommandInteractionDataResolved>,
     ) -> Result<Self, ParseOptionErrorType> {
-        let role_id: RoleId = match value {
+        let role_id = match value {
             CommandOptionValue::Role(value) => value,
             other => return Err(ParseOptionErrorType::InvalidType(other.kind())),
         };
